@@ -368,7 +368,7 @@ class TestRegistry(unittest.TestCase):
         registry = self._makeOne()
         registry.register('test', 'registered', 'abc', name='yup')
         class ABC:
-            __component_types__ = ('abc',)
+            __inherited_component_types__ = ('abc',)
         abc = ABC()
         result = registry.resolve('test', abc, name='yup')
         self.assertEqual(result, 'registered')
@@ -755,6 +755,16 @@ class TestProvidedBy(unittest.TestCase):
         result = self._callFUT(foo)
         self.assertEqual(list(result), [Foo, None])
 
+    def test_obj_is_onlyprovides(self):
+        from repoze.component import provides
+        from repoze.component import onlyprovides
+        class Foo(object):
+            provides('a', 'b')
+        foo = Foo()
+        onlyprovides(foo, 'z')
+        result = self._callFUT(foo)
+        self.assertEqual(list(result), ['z', type(foo), None])
+
     def test_string(self):
         result = self._callFUT('foo')
         self.assertEqual(list(result), [str, None])
@@ -814,7 +824,8 @@ class TestProvidesInsideClass(unittest.TestCase):
             provides('abc', 'def')
         class Foo2(Foo):
             provides('ghi')
-        self.assertEqual(Foo2.__component_types__, ('ghi', 'abc', 'def'))
+        self.assertEqual(Foo2.__inherited_component_types__,
+                         ('ghi', 'abc', 'def'))
 
     def test_morethanonce(self):
         from repoze.component import provides
@@ -826,6 +837,47 @@ class TestProvidesInsideClass(unittest.TestCase):
                 pass
             else:
                 raise AssertionError('wrong')
+
+class TestOnlyProvidesAsFunction(unittest.TestCase):
+    def _callFUT(self, obj, *types):
+        from repoze.component import onlyprovides
+        onlyprovides(obj, *types)
+
+    def test_multiple_calls(self):
+        class Foo(object):
+            __component_types__ = ('a',)
+        foo = Foo()
+        self._callFUT(foo, 'abc', 'def')
+        self._callFUT(foo, 'ghi')
+        self.assertEqual(foo.__component_types__, ('ghi',))
+
+    def test_no_types(self):
+        from repoze.component import onlyprovides
+        self.assertRaises(TypeError, provides)
+
+class TestProvidesInsideClass(unittest.TestCase):
+    def test_class_inheritance(self):
+        from repoze.component import provides
+        from repoze.component import onlyprovides
+        class Foo(object):
+            provides('abc', 'def')
+        class Foo2(Foo):
+            onlyprovides('ghi')
+        self.assertEqual(Foo2.__inherited_component_types__,
+                         ('ghi',))
+        self.assertEqual(Foo.__inherited_component_types__,
+                         ('abc', 'def'))
+
+    def test_morethanonce(self):
+        from repoze.component import onlyprovides
+        class Foo(object):
+            onlyprovides('abc', 'def')
+            try:
+                onlyprovides('ghi')
+            except TypeError:
+                pass
+            else:
+                raise AssertionError('wrong')
         
 class TestAugmentedProduct(unittest.TestCase):
     def _callFUT(self, arg, default_list):
